#ifndef GLOBAL_H
#define GLOBAL_H

#include <nokix/all.h>

/*
** global.h
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2.1 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

/* skin description structure */
struct skin
{
 void *background;
 STRPTR digital_font;
 STRPTR weekday_font;
 STRPTR date_font;
 STRPTR temp_font;
 ULONG flags;
 UBYTE background_x, background_y;
 UBYTE digital_x, digital_y;
 UBYTE weekday_x, weekday_y;
 UBYTE date_x, date_y;
 UBYTE analog_x, analog_y;
 UBYTE temp_x, temp_y;
};

/* skin flags */
#define CIF_BACKGROUND_ANIM			(0x00000001)
#define CIF_DIGITAL					(0x00000002)
#define CIF_DIGITAL_RIGHT			(0x00000004)
#define CIF_DIGITAL_CENTER			(0x00000008)
#define CIF_DIGITAL_XOR				(0x00000010)
#define CIF_WEEKDAY					(0x00000020)
#define CIF_WEEKDAY_RIGHT			(0x00000040)
#define CIF_WEEKDAY_CENTER			(0x00000080)
#define CIF_WEEKDAY_XOR				(0x00000100)
#define CIF_WEEKDAY_SHORT			(0x00000200)
#define CIF_DATE					(0x00000400)
#define CIF_DATE_RIGHT				(0x00000800)
#define CIF_DATE_CENTER				(0x00001000)
#define CIF_DATE_XOR				(0x00002000)
#define CIF_DATE_DDMMYYYY			(0x00004000)
#define CIF_DATE_DDMM				(0x00008000)
#define CIF_ANALOG					(0x00010000)
#define CIF_ANALOG_XOR				(0x00020000)
#define CIF_POWER_SAVE_CLEAR        (0x00040000)
#define CIF_TEMPERATURE				(0x00080000)
#define CIF_TEMPERATURE_RIGHT			(0x00100000)
#define CIF_TEMPERATURE_CENTER			(0x00200000)
#define CIF_TEMPERATURE_XOR			(0x00400000)
#define CIF_TEMPERATURE_SHORT			(0x00800000)
#define CIF_TEMPERATURE_UNIT			(0x01000000)

/* animation timer message */
#define OMSG_NEXT_FRAME				(0x1e10)

/* screen buffer size in bytes */
#define SCREEN_SIZE					(84*48/8)

/* in this struct we will keep track of screen saver when it is opened */
struct data
{
   unsigned short skin;          /* current skin index */
   short frame;                  /* current animation frame index, unused for static skins */
   bool power_save;              /* power save mode status */
   bool negative;                /* negative mode */
   char pad0[2];

 UBYTE screen[SCREEN_SIZE];      /* screen buffer for double buffered animations drawing */
};

/* defined in global.c, allocated when screen saver is opened */
extern struct data *data;

/* skins table, created by REXX script */
extern struct skin *skins[];

void freegfx_draw(void); /* screen saver draw function (from freegfx_draw.c) */

#endif /* GLOBAL_H */
