
#include <nokix/all.h>

/*
** screen_saver.c
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2.1 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "global.h"

#define OMSG_SCREEN_SAVER			(0x1e11)
#define OMSG_SAVE_POWER				(0x1e12)

REXX_EXPOSE(OMSG_SCREEN_SAVER);

extern rexx_uint confid_main;
extern rexx_uint confid_timeout;
extern rexx_uint confid_power_save;
extern rexx_uint confid_negative;
extern rexx_uint confid_skin;
extern rexx_uint confid_skins;

extern rexx_uint num_of_skins;
extern rexx_uint id_lights_timer;

extern UBYTE bars_status[];

static const ULONG tim_anim=OMSG_NEXT_FRAME;
static const ULONG tim_start=OMSG_SCREEN_SAVER;
static const ULONG tim_power_save=OMSG_SAVE_POWER;
static const UWORD timeout[]={5,20,60,120,300,600}; /* in seconds */
static const UBYTE power_save_timeout[]={0,2,6,30}; /* seconds = x*10, first (0) unused */

static BOOL dont_open;	/* = FALSE */

/**************************************************************************************/

void ss_draw(void);

/* screen saver messages dispatcher */
ULONG screen_saver_handler(ULONG msg)
{
 switch(msg)
 {
  case 0x5f2: /* comes when some text is removed from place of operator name */
    dont_open=FALSE;
  case MSG_D_QUIT:
  case MSG_D_QUIT_NOW:
  case MSG_D_INIT: /* comes at startup and when menu is leaved */
  case MSG_SHOW_PENDING_DISP_MSGS:
  case MSG_KEYRELEASE:
  {							/* START SCREEN SAVER TIMER */
   own_timer_abort(&tim_start);
   if(!data) /* if not opened */
   {
    if(own_get_profile(confid_main,current_profile))
      own_timer_start(&tim_start,TT_MESSAGE,TICKS_PER_SECOND*timeout[own_get_profile(confid_timeout,current_profile)]);
   }
   break;
  }
  case 0x5f0: /* comes when some text is put in place of operator name */
   dont_open=TRUE;
  case MSG_START_HANDLER:
  case MSG_DISPLAY_MESSAGE:
  case MSG_KEYPRESS:
  case 0x231:  // profile changed?
  {							/* CLOSE SCREEN SAVER */
   if(data) /* if opened */
   {
    own_timer_abort(&tim_anim);
    own_timer_abort(&tim_power_save);

    freegfx_close();
    free(data); data=NULL;
   }
   if( msg == 0x231 ) send_message(MSG_SHOW_PENDING_DISP_MSGS);   // restart timeout
   break;
  }
  case OMSG_SCREEN_SAVER: /* OPEN SCREEN SAVER */
  {
   msg=MSG_NOP;
   if(msg_argc==1)
   {
    send_message(MSG_CLOSE_DISPLAY_MESSAGE);
    send_message(MSG_LIGHTS_ON);
    send_message(MSG_START_TIMER|MSG_2P,id_lights_timer,(unsigned)(15*TICKS_PER_SECOND));
    send_message(OMSG_SCREEN_SAVER);
    break;
   }
   if((!data)&&
      (!dont_open)&&
      (call_status==0)&&
      ((dial_number_buf[0]=='\0')||(keylock_status==KEYLOCK_STATUS_LOCKED))&&
      (netmonitor_status==0)
      && ( get_phone_state() == PHST_POWER_ON_NORMAL ) )
   {
    UWORD skin;
    
    if(confid_skin!=0xff) /* if more than one skin */
	{
	 skin=own_get_profile(confid_skin,current_profile);
     if(skin>=num_of_skins) skin=0;
    }
    else skin=0; /* we have only one skin */
	
	/* check if clock is set or we have a skin without clock */
    if((get_timestamp()!=TIMESTAMP_UNSET)||(!(skins[skin]->flags&(CIF_DIGITAL|CIF_WEEKDAY|CIF_DATE|CIF_ANALOG))))
    {
     data=malloc(sizeof(struct data));
	 data->skin=skin;

     bars_status[0]=0; /* don't refresh battery bars */
     bars_status[1]=0; /* don't refresh network bars */
	 freegfx_open();
	 
	 own_screen_set(data->screen,0);
	 
	 if(skins[skin]->flags&CIF_BACKGROUND_ANIM) /* start animation */
	 {
	  UBYTE pst;

	  own_timer_start(&tim_anim,TT_MESSAGE,(((struct own_anim_frame *)(skins[skin]->background))[0]).time);
	  
	  /* set power save timer to tell us when anim should be stopped */
	  if((pst=power_save_timeout[own_get_config(confid_power_save)]))
	    own_timer_start(&tim_power_save,TT_MESSAGE,pst*10*TICKS_PER_SECOND);

      data->frame=0;
     }
     
     data->power_save=FALSE;
     data->negative=own_get_profile(confid_negative,current_profile);

     ss_draw();

     /* this will cause the screen to be refreshed by UI task, this removes trash after battery bars
	    (bars use diffrent screen refreshing methods and cause trash to be displayed sometimes then
		freegfx_end(TRUE) is used for the first time after freegfx_open() when we are in standby mode) */
	 freegfx_begin(FALSE);
	 freegfx_end(FALSE);
	}
   }
   break;
  }
  case MSG_TIMESTAMP: /* comes every minute */
  {
   LONG t;

   if(data)
   {
    if(skins[data->skin]->flags&CIF_BACKGROUND_ANIM)
    {
     t=own_timer_query(&tim_anim);
     if((t>=0)&&(t<=75)) break; /* less than half second to next frame -> don't bother */
    }
    ss_draw();
   }
   break;
  }
  case OMSG_NEXT_FRAME:
  {
   if(data) /* if opened */
   {
    if(!(((struct own_anim_frame *)(skins[data->skin]->background))[++data->frame]).bitmap) data->frame=0;
   
	if((data->frame!=0)||(!data->power_save))
	{
 	 /* next frame timer */
     own_timer_start(&tim_anim,TT_MESSAGE,(((struct own_anim_frame *)(skins[data->skin]->background))[data->frame]).time);
    }

    ss_draw(); /* draw next frame */
   }
   msg=MSG_NOP;
   break;
  }
  case OMSG_SAVE_POWER: /* enable power saving mode */
  {
   if(data) data->power_save=TRUE;
   msg=MSG_NOP;
   //break;
  }
 }
 return msg;
}

